﻿using System;
using System.Collections.Generic;
using System.Reflection;
using Logger;
using Models;
using Quartz;

namespace Main.Base
{
    /// <summary>
    /// 子任务基类
    /// </summary>
    public class BaseJob : ILogger
    {
        /// <summary>
        /// 获取处理地址
        /// </summary>
        /// <returns></returns>
        protected static List<RequectUrlEntity> InitRequectUrlEntityList()
        {
            List<RequectUrlEntity> list = new List<RequectUrlEntity>
            {
                new RequectUrlEntity()
                {
                    TypeName = "双色球",
                    Items = new List<Item>
                    {
                        #region 双色球
		                new Item
                        {
                            Cid = 81,
                            TagId = 0,
                            Name = "双色球杀号",
                            Url = "/zt/ssqsh/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 79,
                            TagId = 0,
                            Name = "双色球预测",
                            Url = "/zt/ssqyc/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 80,
                            TagId = 0,
                            Name = "双色球字谜",
                            Url = "/zt/ssqzm/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 127,
                            TagId = 0,
                            Name = "程远双色球杀号",
                            Url = "/zt/cyssqsh/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 126,
                            TagId = 0,
                            Name = "360双色球杀号",
                            Url = "/zt/360ssqsh/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 21,
                            TagId = 0,
                            Name = "双色球中奖故事",
                            Url = "/zt/ssqzjgs/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 80,
                            TagId = 135,
                            Name = "双色球字谜汇总",
                            Url = "/zt/ssqzmhz/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 129,
                            TagId = 0,
                            Name = "貂蝉双色球杀号",
                            Url = "/zt/dcssqsh/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 0,
                            TagId = 0,
                            Name = "小宋双色球预测",
                            Url = "/zt/xsssqyc/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 131,
                            TagId = 0,
                            Name = "阳光探码图",
                            Url = "/zt/ygtmt/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 0,
                            TagId = 0,
                            Name = "彩民乐",
                            Url = "/zt/cml/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 0,
                            TagId = 0,
                            Name = "全国联销图",
                            Url = "/zt/qglxt/list-{0}.htm"
                        } 
	                #endregion
                    }
                },
                new RequectUrlEntity()
                {
                    TypeName = "福彩3D",
                    Items = new List<Item>
                    {
                        #region 福彩3D
		                new Item
                        {
                            Cid = 99,
                            TagId = 0,
                            Name = "太湖字谜",
                            Url = "/zt/thzm/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 70,
                            TagId = 24,
                            Name = "晚秋和值谜",
                            Url = "/zt/wqhzm/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 70,
                            TagId = 0,
                            Name = "3d字谜",
                            Url = "/zt/3dzm/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 70,
                            TagId = 82,
                            Name = "3d藏机图",
                            Url = "/zt/3dcjt/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 70,
                            TagId = 17,
                            Name = "3d字谜图谜汇总",
                            Url = "/zt/3dzmtmhz/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 69,
                            TagId = 0,
                            Name = "3d预测",
                            Url = "/zt/3dyc/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 72,
                            TagId = 0,
                            Name = "3d杀码",
                            Url = "/zt/3dsm/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 71,
                            TagId = 0,
                            Name = "3d胆码",
                            Url = "/zt/3ddm/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 98,
                            TagId = 0,
                            Name = "马后炮解太湖字谜",
                            Url = "/zt/mhpjthzm/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 132,
                            TagId = 0,
                            Name = "一句定三码",
                            Url = "/zt/yjdsm/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 70,
                            TagId = 17,
                            Name = "3d字谜汇总",
                            Url = "/zt/3dzmhz/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 77,
                            TagId = 0,
                            Name = "3d试机号后分析",
                            Url = "/zt/3dsjhhfx/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 70,
                            TagId = 137,
                            Name = "3d藏机诗",
                            Url = "/zt/3dcjs/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 69,
                            TagId = 1,
                            Name = "于海滨3d预测",
                            Url = "/zt/yhb3dyc/list-{0}.htm"
                        }
	                #endregion
                    }
                },
                new RequectUrlEntity()
                {
                    TypeName = "大乐透",
                    Items = new List<Item>
                    {
                        #region 大乐透
		                new Item
                        {
                            Cid = 16,
                            TagId = 0,
                            Name = "大乐透预测",
                            Url = "/zt/dltyc/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 128,
                            TagId = 0,
                            Name = "大乐透杀号",
                            Url = "/zt/dltsh/list-{0}.htm"
                        } 
	                    #endregion
                    }
                },
                new RequectUrlEntity()
                {
                    TypeName = "排列三",
                    Items = new List<Item>
                    {
                        #region 排列三
		                new Item
                        {
                            Cid = 73,
                            TagId = 0,
                            Name = "排列三预测",
                            Url = "/zt/pl3yc/list-{0}.htm"
                        },
                        new Item
                        {
                            Cid = 74,
                            TagId = 0,
                            Name = "排列三字谜",
                            Url = "/zt/pl3zm/list-{0}.htm"
                        } 
	                    #endregion
                    }
                }
            };

            return list;
        }

        /// <summary>
        /// 初始化发布者ID，用于后续随机给文章赋值发布者ID
        /// </summary>
        /// <returns></returns>
        protected static List<int> InitEditorId()
        {
            List<int> list = new List<int>
            {
                1,2,3,6,8,9,12,13,14,15,16,17,20,21,22,23,24,25,27,28,29,30,32,34,35,37,38,39,283,8607
            };

            return list;
        }

        /// <summary>
        /// 从作业数据地图中获取配置信息
        /// </summary>
        /// <param name="context">作业数据地图</param>
        /// <returns></returns>
        protected JobConfigEntity GetConfigFromDataMap(IJobExecutionContext context)
        {
            JobConfigEntity config = new JobConfigEntity();

            //获取JobDataMap
            JobDataMap datamap = context.JobDetail.JobDataMap;
            //获取JobConfigEntity公共属性
            PropertyInfo[] properties = typeof(JobConfigEntity).GetProperties();
            foreach (PropertyInfo info in properties)
            {
                if (info.PropertyType == typeof(string))
                {
                    info.SetValue(config, datamap.GetString(info.Name), null);
                }
                else if (info.PropertyType == typeof(Int32))
                {
                    info.SetValue(config, datamap.GetInt(info.Name), null);
                }
            }
            return config;
        }

        /// <summary>
        /// 系统日志 对try块进行了封装，无返回值
        /// </summary>
        /// <param name="type"></param>
        /// <param name="desc">描述</param>
        /// <param name="errorHandel">异常处理方式</param>
        /// <param name="tryHandel">调试代码</param>
        /// <param name="catchHandel">异常处理方式</param>
        /// <param name="finallHandel">最终处理方式</param>
        public void Logger(Type type, string desc, Action tryHandel, Action<Exception> catchHandel = null, Action finallHandel = null, ErrorHandel errorHandel = ErrorHandel.Throw)
        {
            LogHelper.Logger(type, desc, errorHandel, tryHandel, catchHandel, finallHandel);
        }
    }
}