﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Text;
using Common;
using Common.Stopwatch;
using HtmlAgilityPack;
using Logger;
using Main.Base;
using Models;
using Quartz;

namespace Main.Job
{
    public class ThematicArticleSpiderJob : BaseJob, IJob
    {
        #region Attribute
        /// <summary>
        /// 当前作业配置信息
        /// </summary>
        private JobConfigEntity _config = null;
        /// <summary>
        /// 日志
        /// </summary>
        private static LogHelper _logHelper = new LogHelper(typeof(ThematicArticleSpiderJob));
        /// <summary>
        /// HttpHelper
        /// </summary>
        private static HttpHelper _helper = new HttpHelper();
        /// <summary>
        /// HtmlDocument
        /// </summary>
        private static HtmlDocument _document = new HtmlDocument();
        /// <summary>
        /// 数据源地址
        /// </summary>
        private static List<RequectUrlEntity> _requectUrlList = InitRequectUrlEntityList();
        #endregion

        /// <summary>
        /// 作业执行入口
        /// </summary>
        /// <param name="context">作业执行上下文</param>
        public void Execute(IJobExecutionContext context)
        {
            Logger(typeof(ThematicArticleSpiderJob), "ThematicArticleSpiderJob-作业执行入口", () =>
            {
                _config = this.GetConfigFromDataMap(context);

                //TODO 保存LastArticleId
                //context.JobDetail.JobDataMap[""] = lastArticleId;

                Trace.WriteLine(_config.Name + "=============开始执行：" + DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"));
                _logHelper.Info(_config.Name + "=============开始执行：" + DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"));

                GetColumnArticleList();

                Trace.WriteLine(_config.Name + "=============执行结束：" + DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"));
                _logHelper.Info(_config.Name + "=============执行结束：" + DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"));

            }, e =>
            {
                Trace.WriteLine("引发异常：" + e.Message);
                _logHelper.Error("引发异常：" + e.Message);
            }, () =>
            {

            }, ErrorHandel.Continue);
        }

        /// <summary>
        /// 获取需要抓取专题文章地址
        /// 地址-标题|Cid
        /// </summary>
        /// <returns></returns>
        private void GetColumnArticleList()
        {
            Dictionary<string, string> dict = new Dictionary<string, string>();
            //主地址
            string mainUrl = _config != null && !string.IsNullOrEmpty(_config.MainUrl) ? _config.MainUrl : "http://www.55123.cn";

            foreach (RequectUrlEntity urls in _requectUrlList)
            {
                Trace.WriteLine($"开始处理【{urls.TypeName}】专题数据。\r\n");
                _logHelper.Info($"开始处理【{urls.TypeName}】专题数据。\r\n");

                List<Item> items = urls.Items;
                foreach (Item item in items)
                {
                    //下载页面，默认全部只读取到第10页
                    for (int i = 1; i < 10; i++)
                    {
                        string sourceurl = $"{mainUrl}" + string.Format(item.Url, i);

                        Trace.WriteLine($"开始处理【{item.Name}】相关数据，页面地址：{sourceurl}。\r\n");
                        _logHelper.Info($"开始处理【{item.Name}】相关数据，页面地址：{sourceurl}。\r\n");

                        string html = GetPageHtml(sourceurl);
                        if (!string.IsNullOrEmpty(html))
                        {
                            _document.LoadHtml(html);
                            // /zt/ssqsh/2396489.htm
                            List<HtmlNode> nodes = _document.DocumentNode.Descendants("a").ToList();
                            foreach (HtmlNode node in nodes)
                            {
                                // /zt/\w+/\d+.htm
                                string href = node.Attributes["href"].Value;
                                string text = node.InnerText;

                                bool isMatch = RegexHelper.IsMatch(href, "/zt/\\w+/\\d+.htm");
                                if (isMatch)
                                {
                                    if (!dict.ContainsKey(href))
                                    {
                                        dict.Add(href, $"{text}|{item.Cid}|{item.TagId}");
                                    }
                                }
                            }
                        }
                        else
                        {
                            Trace.WriteLine($"未获取到【{sourceurl}】页面信息。");
                            _logHelper.Error($"未获取到【{sourceurl}】页面信息。");
                        }
                    }

                    string time = StopwatchHelper.Stopwatch(() =>
                    {
                        List<ArticleInfo> data = GetArticleInfoList(dict);
                        //TODO 保存到数据库
                        int lastArticleId = ImportColumnArticle(data);
                        if (lastArticleId > 0)
                        {
                            Trace.WriteLine($"保存成功！此次保存了{data.Count}条记录");
                            _logHelper.Info($"保存成功！此次保存了{data.Count}条记录");
                        }
                        else
                        {
                            Trace.WriteLine($"数据保存失败。");
                        }
                    });

                    Trace.WriteLine($"处理【{item.Name}】耗时：{time}");
                    _logHelper.Info($"处理【{item.Name}】耗时：{time}");
                }
            }
        }

        /// <summary>
        /// 导入文章
        /// </summary>
        /// <returns></returns>
        private int ImportColumnArticle(List<ArticleInfo> columnArticleList)
        {
            int result = 0;

            if (columnArticleList == null || columnArticleList.Count == 0)
            {
                return -1;
            }

            Trace.WriteLine($"开始保存数据，共有{columnArticleList.Count}条记录。");
            _logHelper.Info($"开始保存数据，共有{columnArticleList.Count}条记录。");

            Logger(this.GetType(), "导入文章-ImportColumnArticle", () =>
            {
                using (SqlConnection sqlConnection = new SqlConnection(SqlHelper.GetDbConnectionString(DbConnectionType.CB55128)))
                {
                    if (sqlConnection.State != ConnectionState.Open)
                    {
                        sqlConnection.Open();
                    }

                    SqlDataAdapter sqlDataAdapter = new SqlDataAdapter
                    {
                        SelectCommand = new SqlCommand(
                       "SELECT TOP 0 [ZT_Article].[Id],[ZT_Article].[Title],[ZT_Article].[Cid],[ZT_Article].[TagId],[ZT_Article].[ShortDetail],[ZT_Article].[Addtime],[ZT_ArticleDetail].[Detail],[ZT_ArticleDetail].[EditId],[ZT_ArticleDetail].[Editor] FROM [ZT_Article] INNER JOIN [ZT_ArticleDetail] ON [ZT_Article].[Id]=[ZT_ArticleDetail].[Id]",
                       sqlConnection),
                        InsertCommand = new SqlCommand("dbo.[cc_columnarticle_import]", sqlConnection)
                    };

                    sqlDataAdapter.InsertCommand.Parameters.Add("@id", SqlDbType.Int, 4, "Id");
                    sqlDataAdapter.InsertCommand.Parameters.Add("@title", SqlDbType.NVarChar, 300, "Title");
                    sqlDataAdapter.InsertCommand.Parameters.Add("@cid", SqlDbType.Int, 4, "Cid");
                    sqlDataAdapter.InsertCommand.Parameters.Add("@tagid", SqlDbType.Int, 4, "TagId");
                    sqlDataAdapter.InsertCommand.Parameters.Add("@shortdetail", SqlDbType.NVarChar, 200, "ShortDetail");
                    sqlDataAdapter.InsertCommand.Parameters.Add("@addtime", SqlDbType.DateTime, 8, "Addtime");
                    sqlDataAdapter.InsertCommand.Parameters.Add("@detail", SqlDbType.NVarChar, 0, "Detail");
                    sqlDataAdapter.InsertCommand.Parameters.Add("@editid", SqlDbType.Int, 4, "EditId");
                    sqlDataAdapter.InsertCommand.Parameters.Add("@editor", SqlDbType.NChar, 10, "Editor");
                    sqlDataAdapter.InsertCommand.UpdatedRowSource = UpdateRowSource.None;
                    sqlDataAdapter.InsertCommand.CommandType = CommandType.StoredProcedure;
                    sqlDataAdapter.UpdateBatchSize = 0;

                    DataSet dataSet = new DataSet();
                    sqlDataAdapter.Fill(dataSet);
                    foreach (ArticleInfo current in columnArticleList)
                    {
                        dataSet.Tables[0].Rows.Add(new object[]
                        {
                            current.Id,
                            current.Title,
                            current.Cid,
                            current.TagId,
                            current.ShortDetail,
                            current.Addtime,
                            current.Detail,
                            current.EditId,
                            current.Editor
                        });
                        //result = current.Id;

                        Trace.WriteLine($"正在保存{current.Title}");
                    }

                    result = sqlDataAdapter.Update(dataSet.Tables[0]);

                    sqlDataAdapter.Dispose();
                    dataSet.Clear();
                    dataSet.Dispose();
                    sqlConnection.Close();
                }
            }, e =>
            {
                Trace.WriteLine($"保存数据发生异常，异常信息：{e.Message}");
                _logHelper.Error($"保存数据发生异常，异常信息：{e.Message}");
            });
            return result;
        }

        /// <summary>
        /// 获取文章信息
        /// </summary>
        /// <returns></returns>
        private List<ArticleInfo> GetArticleInfoList(Dictionary<string, string> dict)
        {
            List<ArticleInfo> list = new List<ArticleInfo>();

            if (dict.Count > 0)
            {
                foreach (KeyValuePair<string, string> pair in dict)
                {
                    string url = pair.Key;
                    string title = pair.Value;

                    string time = StopwatchHelper.Stopwatch(() =>
                    {
                        ArticleInfo info = GetArticleInfo(url, title);

                        if (info != null)
                        {
                            if (info.Addtime.ToString("yyyyMMdd") == DateTime.Now.ToString("yyyyMMdd"))
                            {
                                list.Add(info);
                            }
                            
                        }
                    });

                    Trace.WriteLine($"处理文章标题为：【{title}】,地址为：{url}，耗时：{time}");
                    _logHelper.Info($"处理文章标题为：【{title}】,地址为：{url}，耗时：{time}");
                }
            }
            return list;
        }

        /// <summary>
        /// 获取文章信息
        /// </summary>
        /// <param name="url"></param>
        /// <param name="title"></param>
        /// <returns></returns>
        private ArticleInfo GetArticleInfo(string url, string title)
        {
            ArticleInfo info = new ArticleInfo();
            //主地址
            string mainUrl = _config != null && !string.IsNullOrEmpty(_config.MainUrl) ? _config.MainUrl : "http://www.55123.cn";
            //资源地址
            string sourceurl = $"{mainUrl}{url}";

            Trace.WriteLine($"开始处理：{sourceurl}，文章标题为：{title}");
            _logHelper.Info($"开始处理：{sourceurl}，文章标题为：{title}");

            //获取详细页面内容
            string html = GetPageHtml(sourceurl);
            if (!string.IsNullOrEmpty(html))
            {
                _document.LoadHtml(html);

                /**
                 * 匹配到了ID才继续后面的操作
                 */

                //匹配ID
                // /zt/3dzm/2397442.htm
                bool isMatchId = RegexHelper.IsMatch(url, "[1-9]\\d{5,}");
                if (isMatchId)
                {
                    string val = RegexHelper.GetMatchResult(url, "[1-9]\\d{5,}");
                    if (info.Id == 0)
                    {
                        info.Id = Convert.ToInt32(val);
                    }

                    //获取文章详细内容
                    //List<HtmlNode> p_nodes = _document.DocumentNode.Descendants("p").ToList();
                    HtmlNode source_node = _document.DocumentNode.SelectSingleNode("//*[@class=\"text\"]");

                    if (source_node != null)
                    {
                        string source = source_node.InnerHtml;
                        if (!string.IsNullOrEmpty(source))
                        {
                            if (string.IsNullOrEmpty(info.Detail))
                            {
                                info.Detail = source;
                                //简介
                                string shortdetail = source_node.InnerText.Length > 160 ? source_node.InnerText.Substring(0, 159) : source_node.InnerText;
                                info.ShortDetail = shortdetail;
                            }

                            //匹配到文章详细信息才继续

                            //Cid和标题
                            string[] array = title.Split("|".ToCharArray());
                            if (array.Length == 3)
                            {
                                if (string.IsNullOrEmpty(info.Title))
                                {
                                    //标题
                                    info.Title = array[0];
                                    //Cid
                                    info.Cid = Convert.ToInt32(array[1]);
                                    //TagId
                                    info.TagId = Convert.ToInt32(array[2]);
                                }
                            }

                            //作者ID
                            if (info.EditId == 0)
                            {
                                info.EditId = GetEditorId();
                            }

                            //获取所有u标签,用于匹配编辑、更新日期
                            List<HtmlNode> u_nodes = _document.DocumentNode.Descendants("u").ToList();
                            foreach (HtmlNode node in u_nodes)
                            {
                                string text = node.InnerText.Trim();
                                //匹配作者
                                bool isMatchEditor = RegexHelper.IsMatch(text, "[A-Za-z0-9_\\-\u4e00-\u9fa5]+");
                                if (isMatchEditor)
                                {
                                    if (string.IsNullOrEmpty(info.Editor))
                                        info.Editor = text;
                                }
                                //匹配发布日期
                                bool isMatchUpdateTime = RegexHelper.IsMatch(text, "\\d{4}(\\-|\\/|.)\\d{1,2}\\1\\d{1,2}");
                                if (isMatchUpdateTime)
                                {
                                    if (info.Addtime != null)
                                    {
                                        text = $"{text} {DateTime.Now:HH:mm:ss}";
                                        DateTime addtime = Convert.ToDateTime(text);
                                        if (addtime > DateTime.MinValue && addtime < DateTime.MaxValue)
                                        {
                                            info.Addtime = addtime;
                                        }
                                        else
                                        {
                                            info.Addtime = DateTime.Now;
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            info = null;
                        }
                    }
                }
            }

            return info;
        }

        /// <summary>
        /// 获取作者ID
        /// </summary>
        /// <returns></returns>
        private int GetEditorId()
        {
            List<int> list = InitEditorId();
            Random random = new Random();
            int i = random.Next(list.Count);

            return list[i];
        }

        /// <summary>
        /// 下载网页
        /// </summary>
        /// <returns></returns>
        private string GetPageHtml(string url)
        {
            HttpItem item = new HttpItem
            {
                Url = url,
                Method = "GET",
                Accept = "text/html",
                ContentType = "text/html",
                Timeout = 2 * 60 * 1000,
                Encoding = Encoding.UTF8,
                IsToLower = true
            };

            HttpResult result = _helper.GetHtml(item);
            if (result.StatusCode == HttpStatusCode.OK)
            {
                string html = result.Html;
                if (!string.IsNullOrEmpty(html))
                {
                    return html;
                }
            }
            return "";
        }
    }
}