﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Main.Lottery
{
    /// <summary>
    /// 通用方法
    /// </summary>
    public static class EnumFunctionType
    {
        /// <summary>
        /// 第一位，总数，第二位 红球，第三位蓝球,不在这的是全红或者其他格式
        /// </summary>
        private static readonly Dictionary<string, int[]> LotteryBallTypeDict = new Dictionary<string, int[]>() {
            { "QGC_SSQ",new int[3] {7,6,1 }},
            { "QGC_FC3D",new int[3] {3,3,0}},
            { "QGC_DLT",new int[3]{7,5,2 }},
            { "QGC_PL3",new int[3]{ 3,3,0}},
            { "QGC_PL5",new int[3]{ 5,5,0} },
            { "QGC_QXC",new int[3]{ 7,7,0}},
            { "QGC_QLC",new int[3]{8,7,1 } },
            { "DFC_HC1",new int[3]{1,1,0 } },
            { "DFC_DF6J1",new int[3]{ 7,6,1}},
            { "DFC_ZJTC6J1",new int[3]{7,6,1 }},
            { "DFC_HD15X5",new int[3]{5,5,0 } },
            { "DFC_JSTC7WS",new int[3]{7,7,0 }},
            { "DFC_HB30X5",new int[3]{ 5,5,0} },
            { "DFC_HN22X5",new int[3]{ 5,5,0} },
            { "DFC_XJ35X7",new int[3]{8,7,1 } },
            { "DFC_FJ36X7",new int[3]{8,7,1 } },
            { "DFC_NYFC36X7",new int[3]{7,6,1 } },
            { "DFC_FJ31X7",new int[3]{ 8,7,1} },
            { "GPC_BJKL8",new int[3]{21,20,1 } }};

        /// <summary>
        /// 彩票名字和简写对应
        /// </summary>
        private static readonly Dictionary<string, string> TypeOfNameDict = new Dictionary<string, string>() {
            { "QGC_SSQ","双色球" },
            { "QGC_FC3D","福彩3D"},
            { "QGC_DLT","大乐透"},
            { "QGC_PL3","排列三"},
            { "QGC_PL5","排列五"},
            { "QGC_QXC","七星彩"},
            { "QGC_QLC","七乐彩"}};
        
        /// <summary>
        /// 根据彩票类型取得彩票名字
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public static string GetLotteryName(string key)
        {
            if (TypeOfNameDict.Keys.Contains(key))
            {
                return TypeOfNameDict[key];
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// 跟据类型名获取球类型
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public static LotteryBallType GetBallType(string key)
        {
            try
            {
                if (LotteryBallTypeDict.Keys.Contains(key))
                {
                    int[] value = LotteryBallTypeDict[key];
                    LotteryBallType rttype = new LotteryBallType()
                    {
                        Count = value[0],
                        RedCount = value[1],
                        BlueCount = value[2]
                    };
                    return rttype;
                }

                return null;
            }
            catch (Exception)
            {
                return null;
            }
        }
    }

    /// <summary>
    /// 球类别
    /// </summary>
    public class LotteryBallType
    {
        /// <summary>
        /// 总数
        /// </summary>
        public int Count { get; set; }
        /// <summary>
        /// 红球个数
        /// </summary>
        public int RedCount { get; set; }
        /// <summary>
        /// 蓝球个数
        /// </summary>
        public int BlueCount { get; set; }
    }
}