﻿using System;
using System.Collections.Generic;
using System.Linq;
using Common;
using Models;

namespace Main.Lottery
{
    /// <summary>
    /// 处理彩种
    /// </summary>
    public static class LottryDataHandle
    {
        /// <summary>
        /// 组装视图彩种模型
        /// </summary>
        /// <param name="id">ID</param>
        /// <param name="typename">类型名</param>
        /// <param name="term">期数</param>
        /// <param name="opentime">开奖时间</param>
        /// <param name="openCode">开奖号</param>
        /// <param name="detail">开奖详情没有传""或者null</param>
        /// <param name="menuName">类别</param>
        /// <returns></returns>
        public static LotteryData GetLotteryData(SCCLottery typename, int term, DateTime opentime, string id, IList<int> openCode, string detail, string menuName = "")
        {
            try
            {
                LotteryData lotteryData = new LotteryData
                {
                    Term = term.ToString(),
                    OpenTime = opentime.ToString("yyyy-MM-dd HH:mm"),
                    LotteryName = typename.GetEnumDescription(),
                    LotteryType = typename,
                    MenuName = menuName,
                    ID = id,
                    Details = new List<LotteryDetail>()
                };

                LotteryBallType datatype = EnumFunctionType.GetBallType(typename.ToString());

                if (datatype != null)
                {
                    lotteryData.BallCount = datatype.Count;
                    lotteryData.RedBall = Array.ConvertAll<int, string>(openCode.Take(datatype.RedCount).ToArray(), s => s.ToString());
                    if (datatype.BlueCount > 0)
                    {
                        lotteryData.BlueBall = Array.ConvertAll<int, string>(openCode.Skip(datatype.RedCount).ToArray(),s => s.ToString());
                    }

                    if (!string.IsNullOrEmpty(detail)) //开奖详情内容
                    {
                        KaijiangDetailsEntity data = detail.JsonToEntity<KaijiangDetailsEntity>();

                        if (data != null)
                        {
                            if (!string.IsNullOrEmpty(data.Trje))
                            {
                                lotteryData.MoneyCount = data.Trje.Replace(",", "");
                            }

                            if (!string.IsNullOrEmpty(data.Gdje))
                            {
                                lotteryData.MoneyPond = data.Gdje.Replace(",", "");
                            }

                            //int takenum = EnumFunctionType.ContainDetailType.Contains(typename) ? 1 : 2;

                            List<LotteryResults> list = new List<LotteryResults>();
                            if (data.KaiJiangItems != null)
                            {
                                foreach (Kaijiangitem item in data.KaiJiangItems)
                                {
                                    int.TryParse(item.Total, out int couttotal);

                                    LotteryResults tmpresults = new LotteryResults()
                                    {
                                        LevelName = item.Name,
                                        Count = couttotal,
                                        Money = item.TotalMoney
                                    };
                                    list.Add(tmpresults);
                                }
                            }
                            lotteryData.ResultsList = list;
                        }
                    }
                    return lotteryData;
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                return null;
            }

            return null;
        }
    }
}